@@ -248,6 +248,9 @@ void
 virCapabilitiesFreeMachines(virCapsGuestMachinePtr *machines,
                             int nmachines);
 
+void
+virCapabilitiesFreeGuest(virCapsGuestPtr guest);
+
 virCapsGuestPtr
 virCapabilitiesAddGuest(virCapsPtr caps,
                         int ostype,
@@ -294,4 +297,10 @@ virCapabilitiesFormatXML(virCapsPtr caps);
 virBitmapPtr virCapabilitiesGetCpusForNodemask(virCapsPtr caps,
                                                virBitmapPtr nodemask);
 
+int virCapabilitiesGetNodeInfo(virNodeInfoPtr nodeinfo);
+
+int virCapabilitiesInitPages(virCapsPtr caps);
+
+int virCapabilitiesInitNUMA(virCapsPtr caps);
+
 #endif /* __VIR_CAPABILITIES_H */
