@@ -28,10 +28,11 @@
 # include "internal.h"
 # include "virbitmap.h"
 # include "virutil.h"
-# include "virthread.h"
+# include "virscsihost.h"
 # include "virpci.h"
+# include "virvhba.h"
 # include "device_conf.h"
-# include "object_event.h"
+# include "storage_adapter_conf.h"
 
 # include <libxml/tree.h>
 
@@ -105,110 +106,164 @@ typedef enum {
 
 VIR_ENUM_DECL(virNodeDevDRM)
 
-typedef struct _virNodeDevCapData {
+typedef struct _virNodeDevCapSystemHardware virNodeDevCapSystemHardware;
+typedef virNodeDevCapSystemHardware *virNodeDevCapSystemHardwarePtr;
+struct _virNodeDevCapSystemHardware {
+    char *vendor_name;
+    char *version;
+    char *serial;
+    unsigned char uuid[VIR_UUID_BUFLEN];
+};
+
+typedef struct _virNodeDevCapSystemFirmware virNodeDevCapSystemFirmware;
+typedef virNodeDevCapSystemFirmware *virNodeDevCapSystemFirmwarePtr;
+struct _virNodeDevCapSystemFirmware {
+    char *vendor_name;
+    char *version;
+    char *release_date;
+};
+
+typedef struct _virNodeDevCapSystem virNodeDevCapSystem;
+typedef virNodeDevCapSystem *virNodeDevCapSystemPtr;
+struct _virNodeDevCapSystem {
+    char *product_name;
+    virNodeDevCapSystemHardware hardware;
+    virNodeDevCapSystemFirmware firmware;
+};
+
+typedef struct _virNodeDevCapPCIDev virNodeDevCapPCIDev;
+typedef virNodeDevCapPCIDev *virNodeDevCapPCIDevPtr;
+struct _virNodeDevCapPCIDev {
+    unsigned int domain;
+    unsigned int bus;
+    unsigned int slot;
+    unsigned int function;
+    unsigned int product;
+    unsigned int vendor;
+    unsigned int class;
+    char *product_name;
+    char *vendor_name;
+    virPCIDeviceAddressPtr physical_function;
+    virPCIDeviceAddressPtr *virtual_functions;
+    size_t num_virtual_functions;
+    unsigned int max_virtual_functions;
+    unsigned int flags;
+    virPCIDeviceAddressPtr *iommuGroupDevices;
+    size_t nIommuGroupDevices;
+    unsigned int iommuGroupNumber;
+    int numa_node;
+    virPCIEDeviceInfoPtr pci_express;
+    int hdrType; /* enum virPCIHeaderType or -1 */
+};
+
+typedef struct _virNodeDevCapUSBDev virNodeDevCapUSBDev;
+typedef virNodeDevCapUSBDev *virNodeDevCapUSBDevPtr;
+struct _virNodeDevCapUSBDev {
+   unsigned int bus;
+   unsigned int device;
+   unsigned int product;
+   unsigned int vendor;
+   char *product_name;
+   char *vendor_name;
+};
+
+typedef struct _virNodeDevCapUSBIf virNodeDevCapUSBIf;
+typedef virNodeDevCapUSBIf *virNodeDevCapUSBIfPtr;
+struct _virNodeDevCapUSBIf {
+    unsigned int number;
+    unsigned int _class;		/* "class" is reserved in C */
+    unsigned int subclass;
+    unsigned int protocol;
+    char *description;
+};
+
+typedef struct _virNodeDevCapNet virNodeDevCapNet;
+typedef virNodeDevCapNet *virNodeDevCapNetPtr;
+struct _virNodeDevCapNet {
+    char *address;
+    unsigned int address_len;
+    char *ifname;
+    virNetDevIfLink lnk;
+    virNodeDevNetCapType subtype;  /* LAST -> no subtype */
+    virBitmapPtr features; /* enum virNetDevFeature */
+};
+
+typedef struct _virNodeDevCapSCSIHost virNodeDevCapSCSIHost;
+typedef virNodeDevCapSCSIHost *virNodeDevCapSCSIHostPtr;
+struct _virNodeDevCapSCSIHost {
+    unsigned int host;
+    int unique_id;
+    char *wwnn;
+    char *wwpn;
+    char *fabric_wwn;
+    unsigned int flags;
+    int max_vports;
+    int vports;
+};
+
+typedef struct _virNodeDevCapSCSITarget virNodeDevCapSCSITarget;
+typedef virNodeDevCapSCSITarget *virNodeDevCapSCSITargetPtr;
+struct _virNodeDevCapSCSITarget {
+    char *name;
+};
+
+typedef struct _virNodeDevCapSCSI virNodeDevCapSCSI;
+typedef virNodeDevCapSCSI *virNodeDevCapSCSIPtr;
+struct _virNodeDevCapSCSI {
+    unsigned int host;
+    unsigned int bus;
+    unsigned int target;
+    unsigned int lun;
+    char *type;
+};
+
+typedef struct _virNodeDevCapStorage virNodeDevCapStorage;
+typedef virNodeDevCapStorage *virNodeDevCapStoragePtr;
+struct _virNodeDevCapStorage {
+    unsigned long long size;
+    unsigned long long num_blocks;
+    unsigned long long logical_block_size;
+    unsigned long long removable_media_size;
+    char *block;
+    char *bus;
+    char *drive_type;
+    char *model;
+    char *vendor;
+    char *serial;
+    char *media_label;
+    unsigned int flags;	/* virNodeDevStorageCapFlags bits */
+};
+
+typedef struct _virNodeDevCapSCSIGeneric virNodeDevCapSCSIGeneric;
+typedef virNodeDevCapSCSIGeneric *virNodeDevCapSCSIGenericPtr;
+struct _virNodeDevCapSCSIGeneric {
+    char *path;
+};
+
+typedef struct _virNodeDevCapDRM virNodeDevCapDRM;
+typedef virNodeDevCapDRM *virNodeDevCapDRMPtr;
+struct _virNodeDevCapDRM {
+    virNodeDevDRMType type;
+};
+
+typedef struct _virNodeDevCapData virNodeDevCapData;
+typedef virNodeDevCapData *virNodeDevCapDataPtr;
+struct _virNodeDevCapData {
     virNodeDevCapType type;
     union {
-        struct {
-            char *product_name;
-            struct {
-                char *vendor_name;
-                char *version;
-                char *serial;
-                unsigned char uuid[VIR_UUID_BUFLEN];
-            } hardware;
-            struct {
-                char *vendor_name;
-                char *version;
-                char *release_date;
-            } firmware;
-        } system;
-        struct {
-            unsigned int domain;
-            unsigned int bus;
-            unsigned int slot;
-            unsigned int function;
-            unsigned int product;
-            unsigned int vendor;
-            unsigned int class;
-            char *product_name;
-            char *vendor_name;
-            virPCIDeviceAddressPtr physical_function;
-            virPCIDeviceAddressPtr *virtual_functions;
-            size_t num_virtual_functions;
-            unsigned int max_virtual_functions;
-            unsigned int flags;
-            virPCIDeviceAddressPtr *iommuGroupDevices;
-            size_t nIommuGroupDevices;
-            unsigned int iommuGroupNumber;
-            int numa_node;
-            virPCIEDeviceInfoPtr pci_express;
-            int hdrType; /* enum virPCIHeaderType or -1 */
-        } pci_dev;
-        struct {
-            unsigned int bus;
-            unsigned int device;
-            unsigned int product;
-            unsigned int vendor;
-            char *product_name;
-            char *vendor_name;
-        } usb_dev;
-        struct {
-            unsigned int number;
-            unsigned int _class;		/* "class" is reserved in C */
-            unsigned int subclass;
-            unsigned int protocol;
-            char *description;
-        } usb_if;
-        struct {
-            char *address;
-            unsigned int address_len;
-            char *ifname;
-            virNetDevIfLink lnk;
-            virNodeDevNetCapType subtype;  /* LAST -> no subtype */
-            virBitmapPtr features; /* enum virNetDevFeature */
-        } net;
-        struct {
-            unsigned int host;
-            int unique_id;
-            char *wwnn;
-            char *wwpn;
-            char *fabric_wwn;
-            unsigned int flags;
-            int max_vports;
-            int vports;
-        } scsi_host;
-        struct {
-            char *name;
-        } scsi_target;
-        struct {
-            unsigned int host;
-            unsigned int bus;
-            unsigned int target;
-            unsigned int lun;
-            char *type;
-        } scsi;
-        struct {
-            unsigned long long size;
-            unsigned long long num_blocks;
-            unsigned long long logical_block_size;
-            unsigned long long removable_media_size;
-            char *block;
-            char *bus;
-            char *drive_type;
-            char *model;
-            char *vendor;
-            char *serial;
-            char *media_label;
-            unsigned int flags;	/* virNodeDevStorageCapFlags bits */
-        } storage;
-        struct {
-            char *path;
-        } sg; /* SCSI generic device */
-        struct {
-            virNodeDevDRMType type;
-        } drm;
+        virNodeDevCapSystem system;
+        virNodeDevCapPCIDev pci_dev;
+        virNodeDevCapUSBDev usb_dev;
+        virNodeDevCapUSBIf usb_if;
+        virNodeDevCapNet net;
+        virNodeDevCapSCSIHost scsi_host;
+        virNodeDevCapSCSITarget scsi_target;
+        virNodeDevCapSCSI scsi;
+        virNodeDevCapStorage storage;
+        virNodeDevCapSCSIGeneric sg;
+        virNodeDevCapDRM drm;
     };
-} virNodeDevCapData, *virNodeDevCapDataPtr;
+};
 
 typedef struct _virNodeDevCapsDef virNodeDevCapsDef;
 typedef virNodeDevCapsDef *virNodeDevCapsDefPtr;
@@ -253,65 +308,35 @@ struct _virNodeDeviceObjList {
     virNodeDeviceObjPtr *objs;
 };
 
-typedef struct _virNodeDeviceDriverState virNodeDeviceDriverState;
-typedef virNodeDeviceDriverState *virNodeDeviceDriverStatePtr;
-struct _virNodeDeviceDriverState {
-    virMutex lock;
+char *
+virNodeDeviceDefFormat(const virNodeDeviceDef *def);
 
-    virNodeDeviceObjList devs;		/* currently-known devices */
-    void *privateData;			/* driver-specific private data */
+virNodeDeviceDefPtr
+virNodeDeviceDefParseString(const char *str,
+                            int create,
+                            const char *virt_type);
 
-    /* Immutable pointer, self-locking APIs */
-    virObjectEventStatePtr nodeDeviceEventState;
-};
+virNodeDeviceDefPtr
+virNodeDeviceDefParseFile(const char *filename,
+                          int create,
+                          const char *virt_type);
 
+virNodeDeviceDefPtr
+virNodeDeviceDefParseNode(xmlDocPtr xml,
+                          xmlNodePtr root,
+                          int create,
+                          const char *virt_type);
 
-int virNodeDeviceHasCap(const virNodeDeviceObj *dev, const char *cap);
+int
+virNodeDeviceGetWWNs(virNodeDeviceDefPtr def,
+                     char **wwnn,
+                     char **wwpn);
 
-virNodeDeviceObjPtr virNodeDeviceFindByName(virNodeDeviceObjListPtr devs,
-                                            const char *name);
-virNodeDeviceObjPtr
-virNodeDeviceFindBySysfsPath(virNodeDeviceObjListPtr devs,
-                             const char *sysfs_path)
-    ATTRIBUTE_NONNULL(2);
+void
+virNodeDeviceDefFree(virNodeDeviceDefPtr def);
 
-virNodeDeviceObjPtr virNodeDeviceAssignDef(virNodeDeviceObjListPtr devs,
-                                           virNodeDeviceDefPtr def);
-
-void virNodeDeviceObjRemove(virNodeDeviceObjListPtr devs,
-                            virNodeDeviceObjPtr *dev);
-
-char *virNodeDeviceDefFormat(const virNodeDeviceDef *def);
-
-virNodeDeviceDefPtr virNodeDeviceDefParseString(const char *str,
-                                                int create,
-                                                const char *virt_type);
-virNodeDeviceDefPtr virNodeDeviceDefParseFile(const char *filename,
-                                              int create,
-                                              const char *virt_type);
-virNodeDeviceDefPtr virNodeDeviceDefParseNode(xmlDocPtr xml,
-                                              xmlNodePtr root,
-                                              int create,
-                                              const char *virt_type);
-
-int virNodeDeviceGetWWNs(virNodeDeviceDefPtr def,
-                         char **wwnn,
-                         char **wwpn);
-
-int virNodeDeviceGetParentHost(virNodeDeviceObjListPtr devs,
-                               virNodeDeviceDefPtr def,
-                               int create);
-
-void virNodeDeviceDefFree(virNodeDeviceDefPtr def);
-
-void virNodeDeviceObjFree(virNodeDeviceObjPtr dev);
-
-void virNodeDeviceObjListFree(virNodeDeviceObjListPtr devs);
-
-void virNodeDevCapsDefFree(virNodeDevCapsDefPtr caps);
-
-void virNodeDeviceObjLock(virNodeDeviceObjPtr obj);
-void virNodeDeviceObjUnlock(virNodeDeviceObjPtr obj);
+void
+virNodeDevCapsDefFree(virNodeDevCapsDefPtr caps);
 
 # define VIR_CONNECT_LIST_NODE_DEVICES_FILTERS_CAP \
                 (VIR_CONNECT_LIST_NODE_DEVICES_CAP_SYSTEM        | \
@@ -328,16 +353,16 @@ void virNodeDeviceObjUnlock(virNodeDeviceObjPtr obj);
                  VIR_CONNECT_LIST_NODE_DEVICES_CAP_SCSI_GENERIC  | \
                  VIR_CONNECT_LIST_NODE_DEVICES_CAP_DRM)
 
-typedef bool (*virNodeDeviceObjListFilter)(virConnectPtr conn,
-                                           virNodeDeviceDefPtr def);
+char *
+virNodeDeviceGetParentName(virConnectPtr conn,
+                           const char *nodedev_name);
 
-int virNodeDeviceObjListExport(virConnectPtr conn,
-                               virNodeDeviceObjList devobjs,
-                               virNodeDevicePtr **devices,
-                               virNodeDeviceObjListFilter filter,
-                               unsigned int flags);
+char *
+virNodeDeviceCreateVport(virConnectPtr conn,
+                         virStorageAdapterFCHostPtr fchost);
 
-char *virNodeDeviceGetParentName(virConnectPtr conn,
-                                 const char *nodedev_name);
+int
+virNodeDeviceDeleteVport(virConnectPtr conn,
+                         virStorageAdapterFCHostPtr fchost);
 
 #endif /* __VIR_NODE_DEVICE_CONF_H__ */
