@@ -152,6 +152,20 @@
 #   endif
 #  endif
 
+/**
+ * ATTRIBUTE_NOINLINE:
+ *
+ * Force compiler not to inline a method. Should be used if
+ * the method need to be overridable by test mocks.
+ */
+#  ifndef ATTRIBUTE_NOINLINE
+#   if __GNUC_PREREQ (4, 0)
+#    define ATTRIBUTE_NOINLINE __attribute__((__noinline__))
+#   else
+#    define ATTRIBUTE_NOINLINE
+#   endif
+#  endif
+
 /**
  * ATTRIBUTE_FMT_PRINTF
  *
@@ -236,6 +250,9 @@
 #  ifndef ATTRIBUTE_RETURN_CHECK
 #   define ATTRIBUTE_RETURN_CHECK
 #  endif
+#  ifndef ATTRIBUTE_NOINLINE
+#   define ATTRIBUTE_NOINLINE
+#  endif
 #
 #  ifndef ATTRIBUTE_FALLTHROUGH
 #   define ATTRIBUTE_FALLTHROUGH do {} while(0)
@@ -248,6 +265,10 @@
     _Pragma ("GCC diagnostic push") \
     _Pragma ("GCC diagnostic ignored \"-Wcast-align\"")
 
+#  define VIR_WARNINGS_NO_DEPRECATED \
+    _Pragma ("GCC diagnostic push") \
+    _Pragma ("GCC diagnostic ignored \"-Wdeprecated-declarations\"")
+
 #  if HAVE_SUGGEST_ATTRIBUTE_FORMAT
 #   define VIR_WARNINGS_NO_PRINTF \
     _Pragma ("GCC diagnostic push") \
@@ -272,6 +293,7 @@
     _Pragma ("GCC diagnostic pop")
 # else
 #  define VIR_WARNINGS_NO_CAST_ALIGN
+#  define VIR_WARNINGS_NO_DEPRECATED
 #  define VIR_WARNINGS_NO_PRINTF
 #  define VIR_WARNINGS_NO_WLOGICALOP_EQUAL_EXPR
 #  define VIR_WARNINGS_RESET
