@@ -47,10 +47,10 @@
 #include "viruuid.h"
 #include "virbuffer.h"
 #include "viralloc.h"
-#include "nodeinfo.h"
 #include "virfile.h"
 #include "vircommand.h"
 #include "virstring.h"
+#include "virhostcpu.h"
 
 #define VIR_FROM_THIS VIR_FROM_OPENVZ
 
@@ -165,7 +165,7 @@ virCapsPtr openvzCapsInit(void)
                                    false, false)) == NULL)
         goto no_memory;
 
-    if (nodeCapsInitNUMA(caps) < 0)
+    if (virCapabilitiesInitNUMA(caps) < 0)
         goto no_memory;
 
     if ((guest = virCapabilitiesAddGuest(caps,
@@ -570,7 +570,7 @@ int openvzLoadDomains(struct openvz_driver *driver)
         }
 
         if (ret == 0 || vcpus == 0)
-            vcpus = openvzGetNodeCPUs();
+            vcpus = virHostCPUGetCount();
 
         if (virDomainDefSetVcpusMax(def, vcpus, driver->xmlopt) < 0)
             goto cleanup;
@@ -628,17 +628,6 @@ int openvzLoadDomains(struct openvz_driver *driver)
     return -1;
 }
 
-unsigned int
-openvzGetNodeCPUs(void)
-{
-    virNodeInfo nodeinfo;
-
-    if (nodeGetInfo(&nodeinfo) < 0)
-        return 0;
-
-    return nodeinfo.cpus;
-}
-
 static int
 openvzWriteConfigParam(const char * conf_file, const char *param, const char *value)
 {
