@@ -296,6 +296,8 @@ qemuMonitorPtr qemuMonitorOpenFD(virDomainObjPtr vm,
                                  void *opaque)
     ATTRIBUTE_NONNULL(1) ATTRIBUTE_NONNULL(4);
 
+bool qemuMonitorRegister(qemuMonitorPtr mon)
+    ATTRIBUTE_NONNULL(1);
 void qemuMonitorUnregister(qemuMonitorPtr mon)
     ATTRIBUTE_NONNULL(1);
 void qemuMonitorClose(qemuMonitorPtr mon);
@@ -1067,6 +1069,10 @@ int qemuMonitorGetDeviceAliases(qemuMonitorPtr mon,
 typedef void (*qemuMonitorReportDomainLogError)(qemuMonitorPtr mon,
                                                 const char *msg,
                                                 void *opaque);
+void qemuMonitorSetDomainLogLocked(qemuMonitorPtr mon,
+                                   qemuMonitorReportDomainLogError func,
+                                   void *opaque,
+                                   virFreeCallback destroy);
 void qemuMonitorSetDomainLog(qemuMonitorPtr mon,
                              qemuMonitorReportDomainLogError func,
                              void *opaque,
