@@ -93,8 +93,12 @@ int virStorageBackendDeleteLocal(virConnectPtr conn,
 int virStorageBackendRefreshLocal(virConnectPtr conn,
                                   virStoragePoolObjPtr pool);
 
+int virStorageUtilGlusterExtractPoolSources(const char *host,
+                                            const char *xml,
+                                            virStoragePoolSourceListPtr list,
+                                            virStoragePoolType pooltype);
 int virStorageBackendFindGlusterPoolSources(const char *host,
-                                            int pooltype,
+                                            virStoragePoolType pooltype,
                                             virStoragePoolSourceListPtr list,
                                             bool report);
 
@@ -163,4 +167,8 @@ virStorageBackendCreateQemuImgCmdFromVol(virConnectPtr conn,
 int virStorageBackendSCSIFindLUs(virStoragePoolObjPtr pool,
                                  uint32_t scanhost);
 
+int
+virStorageBackendZeroPartitionTable(const char *path,
+                                    unsigned long long size);
+
 #endif /* __VIR_STORAGE_UTIL_H__ */
