@@ -795,8 +795,8 @@ virHostCPUGetStatsLinux(FILE *procstat,
 
         if (STRPREFIX(buf, cpu_header)) { /* aka logical CPU time */
             if (sscanf(buf,
-                       "%*s %llu %llu %llu %llu %llu" // user ~ iowait
-                       "%llu %llu %llu %llu %llu",    // irq  ~ guest_nice
+                       "%*s %llu %llu %llu %llu %llu" /* user ~ iowait */
+                       "%llu %llu %llu %llu %llu",    /* irq  ~ guest_nice */
                        &usr, &ni, &sys, &idle, &iowait,
                        &irq, &softirq, &steal, &guest, &guest_nice) < 4) {
                 continue;
@@ -847,13 +847,13 @@ virHostCPUParseCountLinux(void)
     tmp = str;
     do {
         if (virStrToLong_i(tmp, &tmp, 10, &ret) < 0 ||
-            !strchr(",-\n", *tmp)) {
+            !strchr(",-", *tmp)) {
             virReportError(VIR_ERR_NO_SUPPORT,
                            _("failed to parse %s"), str);
             ret = -1;
             goto cleanup;
         }
-    } while (*tmp++ != '\n');
+    } while (*tmp++ && *tmp);
     ret++;
 
  cleanup:
