@@ -312,16 +312,19 @@ virMediatedDeviceListDispose(void *obj)
 }
 
 
+/* The reason for @dev to be double pointer is that VIR_APPEND_ELEMENT clears
+ * the pointer and we need to clear the original not a copy on the stack
+ */
 int
 virMediatedDeviceListAdd(virMediatedDeviceListPtr list,
-                         virMediatedDevicePtr dev)
+                         virMediatedDevicePtr *dev)
 {
-    if (virMediatedDeviceListFind(list, dev)) {
+    if (virMediatedDeviceListFind(list, *dev)) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                       _("device %s is already in use"), dev->path);
+                       _("device %s is already in use"), (*dev)->path);
         return -1;
     }
-    return VIR_APPEND_ELEMENT(list->devs, list->count, dev);
+    return VIR_APPEND_ELEMENT(list->devs, list->count, *dev);
 }
 
 
@@ -446,21 +449,26 @@ virMediatedDeviceListMarkDevices(virMediatedDeviceListPtr dst,
 
     virObjectLock(dst);
     for (i = 0; i < count; i++) {
+        const char *mdev_path = NULL;
         virMediatedDevicePtr mdev = virMediatedDeviceListGet(src, i);
 
+        if (!mdev)
+            goto cleanup;
+
+        mdev_path = mdev->path;
         if (virMediatedDeviceIsUsed(mdev, dst) ||
             virMediatedDeviceSetUsedBy(mdev, drvname, domname) < 0)
             goto cleanup;
 
         /* Copy mdev references to the driver list:
-         * - caller is responsible for NOT freeing devices in @list on success
+         * - caller is responsible for NOT freeing devices in @src on success
          * - we're responsible for performing a rollback on failure
          */
-        if (virMediatedDeviceListAdd(dst, mdev) < 0)
+        if (virMediatedDeviceListAdd(dst, &mdev) < 0)
             goto rollback;
 
         VIR_DEBUG("'%s' added to list of active mediated devices used by '%s'",
-                  mdev->path, domname);
+                  mdev_path, domname);
     }
 
     ret = 0;
